@@ -1,21 +1,21 @@
-/* Copyright (C) 1995,1996,1997,1998,1999,2000,2001,2002,2003,2004,2005,2006,2007,2008,2009,2010,2011
- * Free Software Foundation, Inc.
- * 
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
- * the License, or (at your option) any later version.
- *
- * This library is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
- * 02110-1301 USA
- */
+/* Copyright 1995-2011,2018
+     Free Software Foundation, Inc.
+
+   This file is part of Guile.
+
+   Guile is free software: you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as published
+   by the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+
+   Guile is distributed in the hope that it will be useful, but WITHOUT
+   ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+   FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public
+   License for more details.
+
+   You should have received a copy of the GNU Lesser General Public
+   License along with Guile.  If not, see
+   <https://www.gnu.org/licenses/>.  */
 
 
 
@@ -25,45 +25,42 @@
 
 #include <alloca.h>
 
-#include "libguile/__scm.h"
+#include "alist.h"
+#include "async.h"
+#include "continuations.h"
+#include "debug.h"
+#include "deprecation.h"
+#include "dynwind.h"
+#include "eq.h"
+#include "eval.h"
+#include "feature.h"
+#include "fluids.h"
+#include "goops.h"
+#include "gsubr.h"
+#include "hash.h"
+#include "hashtab.h"
+#include "list.h"
+#include "macros.h"
+#include "memoize.h"
+#include "modules.h"
+#include "ports.h"
+#include "print.h"
+#include "procprop.h"
+#include "procs.h"
+#include "programs.h"
+#include "smob.h"
+#include "srcprop.h"
+#include "stackchk.h"
+#include "strings.h"
+#include "threads.h"
+#include "throw.h"
+#include "values.h"
 
-#include "libguile/_scm.h"
-#include "libguile/alist.h"
-#include "libguile/async.h"
-#include "libguile/continuations.h"
-#include "libguile/debug.h"
-#include "libguile/deprecation.h"
-#include "libguile/dynwind.h"
-#include "libguile/eq.h"
-#include "libguile/eval.h"
-#include "libguile/feature.h"
-#include "libguile/fluids.h"
-#include "libguile/goops.h"
-#include "libguile/hash.h"
-#include "libguile/hashtab.h"
-#include "libguile/list.h"
-#include "libguile/macros.h"
-#include "libguile/memoize.h"
-#include "libguile/modules.h"
-#include "libguile/ports.h"
-#include "libguile/print.h"
-#include "libguile/procprop.h"
-#include "libguile/programs.h"
-#include "libguile/root.h"
-#include "libguile/smob.h"
-#include "libguile/srcprop.h"
-#include "libguile/stackchk.h"
-#include "libguile/strings.h"
-#include "libguile/threads.h"
-#include "libguile/throw.h"
-#include "libguile/validate.h"
-#include "libguile/values.h"
-#include "libguile/promises.h"
+#include "promises.h"
 
 
 
 
-
 scm_t_bits scm_tc16_promise;
 
 SCM_DEFINE (scm_make_promise, "make-promise", 1, 0, 0, 
@@ -88,11 +85,11 @@ static int
 promise_print (SCM exp, SCM port, scm_print_state *pstate)
 {
   int writingp = SCM_WRITINGP (pstate);
-  scm_puts_unlocked ("#<promise ", port);
+  scm_puts ("#<promise ", port);
   SCM_SET_WRITINGP (pstate, 1);
   scm_iprin1 (SCM_PROMISE_DATA (exp), port, pstate);
   SCM_SET_WRITINGP (pstate, writingp);
-  scm_putc_unlocked ('>', port);
+  scm_putc ('>', port);
   return !0;
 }
 
@@ -136,14 +133,7 @@ scm_init_promises ()
   scm_tc16_promise = scm_make_smob_type ("promise", 0);
   scm_set_smob_print (scm_tc16_promise, promise_print);
 
-#include "libguile/promises.x"
+#include "promises.x"
 
   scm_add_feature ("delay");
 }
-
-/*
-  Local Variables:
-  c-file-style: "gnu"
-  End:
-*/
-
