@@ -1,5 +1,5 @@
 ;;; Abstract constant folding on CPS
-;;; Copyright (C) 2014, 2015, 2017, 2018 Free Software Foundation, Inc.
+;;; Copyright (C) 2014-2020 Free Software Foundation, Inc.
 ;;;
 ;;; This library is free software: you can redistribute it and/or modify
 ;;; it under the terms of the GNU Lesser General Public License as
@@ -303,6 +303,61 @@
     (lambda (cps k src param arg0 type0 min0 max0 arg1 type1 min1 max1)
       body ...)))
 
+(define (power-of-two? constant)
+  (and (positive? constant)
+       (zero? (logand constant (1- constant)))))
+
+(define-binary-primcall-reducer (quo cps k src param
+                                     arg0 type0 min0 max0
+                                     arg1 type1 min1 max1)
+  (cond
+   ((not (type<=? (logior type0 type1) &exact-integer))
+    (with-cps cps #f))
+   ((and (eqv? type1 &fixnum) (eqv? min1 max1) (power-of-two? min1))
+    (with-cps cps
+      (build-term
+        ($continue k src
+          ($primcall 'rsh/immediate (logcount (1- min1)) (arg0))))))
+   (else
+    (with-cps cps #f))))
+
+(define-binary-primcall-reducer (rem cps k src param
+                                     arg0 type0 min0 max0
+                                     arg1 type1 min1 max1)
+  (cond
+   ((not (type<=? (logior type0 type1) &exact-integer))
+    (with-cps cps #f))
+   ((and (eqv? type1 &fixnum) (eqv? min1 max1) (power-of-two? min1)
+         (<= 0 min0))
+    (with-cps cps
+      (letv mask)
+      (letk kmask
+            ($kargs ('mask) (mask)
+              ($continue k src
+                ($primcall 'logand #f (arg0 mask)))))
+      (build-term
+        ($continue kmask src ($const (1- min1))))))
+   (else
+    (with-cps cps #f))))
+
+(define-binary-primcall-reducer (mod cps k src param
+                                     arg0 type0 min0 max0
+                                     arg1 type1 min1 max1)
+  (cond
+   ((not (type<=? (logior type0 type1) &exact-integer))
+    (with-cps cps #f))
+   ((and (eqv? type1 &fixnum) (eqv? min1 max1) (power-of-two? min1))
+    (with-cps cps
+      (letv mask)
+      (letk kmask
+            ($kargs ('mask) (mask)
+              ($continue k src
+                ($primcall 'logand #f (arg0 mask)))))
+      (build-term
+        ($continue kmask src ($const (1- min1))))))
+   (else
+    (with-cps cps #f))))
+
 (define-unary-primcall-reducer (mul/immediate cps k src constant
                                               arg type min max)
   (cond
@@ -372,6 +427,25 @@
    (else
     (with-cps cps #f))))
 
+(define-binary-primcall-reducer (logior cps k src param
+                                        arg0 type0 min0 max0
+                                        arg1 type1 min1 max1)
+  (cond
+   ((type<=? (logior type0 type1) &exact-integer)
+    (cond
+     ((= 0 min0 max0)
+      (with-cps cps
+        (build-term
+          ($continue k src ($values (arg1))))))
+     ((= 0 min1 max1)
+      (with-cps cps
+        (build-term
+          ($continue k src ($values (arg0))))))
+     (else
+      (with-cps cps #f))))
+   (else
+    (with-cps cps #f))))
+
 (define-unary-primcall-reducer (u64->scm cps k src constant arg type min max)
   (cond
    ((<= max (target-most-positive-fixnum))
